@@ -34,24 +34,29 @@ import (
 )
 
 const (
-	tektonSA         = "tekton-pipelines-controller"
-	CronName         = "tekton-resource-pruner"
-	JobsTKNImageName = "IMAGE_JOB_PRUNER_TKN"
-	pruneSkip        = "operator.tekton.dev/prune.skip"
-	pruneResources   = "operator.tekton.dev/prune.resources"
-	pruneStrategy    = "operator.tekton.dev/prune.strategy"
-	pruneKeep        = "operator.tekton.dev/prune.keep"
-	pruneKeepSince   = "operator.tekton.dev/prune.keep-since"
-	pruneSchedule    = "operator.tekton.dev/prune.schedule"
-	pruneCronLabel   = "tektonconfig.operator.tekton.dev/pruner"
-	pruneCronNsLabel = "tektonconfig.operator.tekton.dev/pruner.ns"
-	keep             = "keep"
-	keepSince        = "keep-since"
+	tektonSA             = "tekton-pipelines-controller"
+	CronName             = "tekton-resource-pruner"
+	JobsTKNImageName     = "IMAGE_JOB_PRUNER_TKN"
+	pruneSkip            = "operator.tekton.dev/prune.skip"
+	pruneResources       = "operator.tekton.dev/prune.resources"
+	pruneStrategy        = "operator.tekton.dev/prune.strategy"
+	pruneKeep            = "operator.tekton.dev/prune.keep"
+	pruneKeepSince       = "operator.tekton.dev/prune.keep-since"
+	pruneSchedule        = "operator.tekton.dev/prune.schedule"
+	pruneCronLabel       = "tektonconfig.operator.tekton.dev/pruner"
+	pruneCronNsLabel     = "tektonconfig.operator.tekton.dev/pruner.ns"
+	pruneLastAppliedHash = "operator.tekton.dev/prune.hash"
+	keep                 = "keep"
+	keepSince            = "keep-since"
+	changed              = true
+	unchanged            = false
 )
 
 type Pruner struct {
-	kc       kubernetes.Interface
-	tknImage string
+	kc              kubernetes.Interface
+	tknImage        string
+	targetNamespace string
+	ownerRef        v1.OwnerReference
 }
 
 type pruneConfigPerNS struct {
@@ -62,14 +67,19 @@ type pruneConfigPerNS struct {
 type pruningNs struct {
 	commonScheduleNs map[string]*pruneConfigPerNS
 	uniqueScheduleNS map[string]*pruneConfigPerNS
+	configChanged    bool
 }
 
 func Prune(ctx context.Context, k kubernetes.Interface, tC *v1alpha1.TektonConfig) error {
-	pruner := &Pruner{kc: k}
+	pruner := &Pruner{
+		kc:              k,
+		targetNamespace: tC.Spec.TargetNamespace,
+		ownerRef:        *v1.NewControllerRef(tC, tC.GetGroupVersionKind()),
+	}
 
 	//to remove cronjob created by older version of operator.
 	oldCronName := CronName[7:]
-	if err := pruner.checkAndDeleteCron(ctx, oldCronName, tC.Spec.TargetNamespace); err != nil {
+	if err := pruner.checkAndDeleteCron(ctx, oldCronName); err != nil {
 		return err
 	}
 
@@ -82,7 +92,7 @@ func Prune(ctx context.Context, k kubernetes.Interface, tC *v1alpha1.TektonConfi
 		}
 		if len(cronJobs.Items) > 0 {
 			for _, cronJob := range cronJobs.Items {
-				if err := pruner.deleteCronJob(ctx, cronJob.Name, cronJob.Namespace); err != nil {
+				if err := pruner.deleteCronJob(ctx, cronJob.Name); err != nil {
 					return err
 				}
 			}
@@ -98,7 +108,7 @@ func Prune(ctx context.Context, k kubernetes.Interface, tC *v1alpha1.TektonConfi
 	}
 	if len(annotationRemovedUniqueCron) > 0 {
 		for _, uniqueCron := range annotationRemovedUniqueCron {
-			if err := pruner.checkAndDeleteCron(ctx, uniqueCron, tC.Spec.TargetNamespace); err != nil {
+			if err := pruner.checkAndDeleteCron(ctx, uniqueCron); err != nil {
 				return err
 			}
 		}
@@ -110,7 +120,6 @@ func Prune(ctx context.Context, k kubernetes.Interface, tC *v1alpha1.TektonConfi
 	}
 	pruner.tknImage = tknImageFromEnv
 	logger := logging.FromContext(ctx)
-	ownerRef := *v1.NewControllerRef(tC, tC.GetGroupVersionKind())
 
 	// for the default config from the tektonconfig
 	pruningNamespaces, err := prunableNamespaces(ctx, k, tC.Spec.Pruner)
@@ -119,8 +128,7 @@ func Prune(ctx context.Context, k kubernetes.Interface, tC *v1alpha1.TektonConfi
 	}
 	if pruningNamespaces.commonScheduleNs != nil && len(pruningNamespaces.commonScheduleNs) > 0 {
 		jobs := pruner.createAllJobContainers(pruningNamespaces.commonScheduleNs)
-		cronNameWithRandomSuffix := SimpleNameGenerator.RestrictLengthWithRandomSuffix(CronName)
-		if err := pruner.checkAndCreate(ctx, cronNameWithRandomSuffix, tC.Spec.TargetNamespace, tC.Spec.TargetNamespace, tC.Spec.Pruner.Schedule, jobs, ownerRef, pruneCronLabel); err != nil {
+		if err := pruner.checkAndCreate(ctx, "", tC.Spec.Pruner.Schedule, jobs, pruneCronLabel, pruningNamespaces.configChanged); err != nil {
 			logger.Error("failed to create cronjob ", err)
 		}
 	}
@@ -128,9 +136,8 @@ func Prune(ctx context.Context, k kubernetes.Interface, tC *v1alpha1.TektonConfi
 	if pruningNamespaces.uniqueScheduleNS != nil {
 		for ns, con := range pruningNamespaces.uniqueScheduleNS {
 			jobs := pruner.createJobContainers(con, ns)
-			cronNameWithRandomSuffix := SimpleNameGenerator.RestrictLengthWithRandomSuffix(CronName)
 			listOpt := pruneCronNsLabel + "=" + ns
-			if err := pruner.checkAndCreate(ctx, cronNameWithRandomSuffix, tC.Spec.TargetNamespace, ns, con.config.Schedule, jobs, ownerRef, listOpt); err != nil {
+			if err := pruner.checkAndCreate(ctx, ns, con.config.Schedule, jobs, listOpt, pruningNamespaces.configChanged); err != nil {
 				logger.Error("failed to create cronjob ", err)
 			}
 		}
@@ -153,13 +160,22 @@ func prunableNamespaces(ctx context.Context, k kubernetes.Interface, defaultPrun
 			continue
 		}
 		nsAnnotations := ns.GetAnnotations()
-		//skip all the namespaces if annotated with prune skip
-		if nsAnnotations[pruneSkip] != "" && nsAnnotations[pruneSkip] == "true" {
-			continue
-		}
+
 		pc := &pruneConfigPerNS{
 			config: v1alpha1.Prune{},
 		}
+		changed, err := diffLastAppliedPruneConfig(ctx, k, defaultPruneConfig, &ns)
+		if err != nil {
+			return prunableNs, err
+		}
+		if changed {
+			prunableNs.configChanged = changed
+		}
+
+		//skip all the namespaces if annotated with prune skip
+		if nsAnnotations[pruneSkip] == "true" {
+			continue
+		}
 
 		if nsAnnotations[pruneResources] != "" {
 			pc.config.Resources = strings.Split(nsAnnotations[pruneResources], ",")
@@ -216,7 +232,6 @@ func prunableNamespaces(ctx context.Context, k kubernetes.Interface, defaultPrun
 				continue
 			}
 		}
-
 		commonSchedule[ns.Name] = pc
 	}
 	prunableNs.commonScheduleNs = commonSchedule
@@ -250,19 +265,30 @@ func (pruner *Pruner) createJobContainers(nsConfig *pruneConfigPerNS, ns string)
 	return containers
 }
 
-func (pruner *Pruner) checkAndCreate(ctx context.Context, cronName, targetNs, uniquePruneNs, schedule string, pruneContainers []corev1.Container, oR v1.OwnerReference, listOpt string) error {
-	cronList, err := pruner.listCronJobs(ctx, targetNs, listOpt)
+func (pruner *Pruner) checkAndCreate(ctx context.Context, uniquePruneNs, schedule string, pruneContainers []corev1.Container, listOpt string, configChanged bool) error {
+	suffixedCronName := SimpleNameGenerator.RestrictLengthWithRandomSuffix(CronName)
+	cronList, err := pruner.listCronJobs(ctx, pruner.targetNamespace, listOpt)
 	if err != nil {
 		return err
 	}
-	if len(cronList.Items) == 1 {
-		if err := pruner.deleteCronJob(ctx, cronList.Items[0].Name, targetNs); err != nil {
-			return err
+	// cronjob exists and change in config
+	if len(cronList.Items) > 0 && configChanged {
+		for _, cronjob := range cronList.Items {
+			if err := pruner.deleteCronJob(ctx, cronjob.Name); err != nil {
+				return err
+			}
 		}
-		return createCronJob(ctx, pruner.kc, cronName, targetNs, uniquePruneNs, schedule, pruneContainers, oR)
+		return createCronJob(ctx, pruner.kc, suffixedCronName, pruner.targetNamespace, uniquePruneNs, schedule, pruneContainers, pruner.ownerRef)
 	}
-	if err := createCronJob(ctx, pruner.kc, cronName, targetNs, uniquePruneNs, schedule, pruneContainers, oR); err != nil {
-		return err
+
+	// no change in config but the cronjob does not exist
+	if len(cronList.Items) == 0 && !configChanged {
+		return createCronJob(ctx, pruner.kc, suffixedCronName, pruner.targetNamespace, uniquePruneNs, schedule, pruneContainers, pruner.ownerRef)
+	}
+
+	// any case with config change
+	if configChanged {
+		return createCronJob(ctx, pruner.kc, suffixedCronName, pruner.targetNamespace, uniquePruneNs, schedule, pruneContainers, pruner.ownerRef)
 	}
 	return nil
 }
@@ -349,12 +375,12 @@ func (pruner *Pruner) listCronJobs(ctx context.Context, ns, option string) (*bat
 	return cronJobs, nil
 }
 
-func (pruner *Pruner) deleteCronJob(ctx context.Context, cronName, ns string) error {
-	return pruner.kc.BatchV1().CronJobs(ns).Delete(ctx, cronName, v1.DeleteOptions{})
+func (pruner *Pruner) deleteCronJob(ctx context.Context, cronName string) error {
+	return pruner.kc.BatchV1().CronJobs(pruner.targetNamespace).Delete(ctx, cronName, v1.DeleteOptions{})
 }
 
-func (pruner *Pruner) checkAndDeleteCron(ctx context.Context, cronName, ns string) error {
-	if _, err := pruner.kc.BatchV1().CronJobs(ns).Get(ctx, cronName, v1.GetOptions{}); err != nil {
+func (pruner *Pruner) checkAndDeleteCron(ctx context.Context, cronName string) error {
+	if _, err := pruner.kc.BatchV1().CronJobs(pruner.targetNamespace).Get(ctx, cronName, v1.GetOptions{}); err != nil {
 		if errors.IsNotFound(err) {
 			return nil
 		}
@@ -362,7 +388,7 @@ func (pruner *Pruner) checkAndDeleteCron(ctx context.Context, cronName, ns strin
 	}
 
 	//if there is no error it means cron does exists, but no prune in config it means delete it
-	return pruner.deleteCronJob(ctx, cronName, ns)
+	return pruner.deleteCronJob(ctx, cronName)
 }
 
 func (pruner *Pruner) removedFromTektonConfig(prune v1alpha1.Prune) bool {
@@ -395,6 +421,54 @@ func (pruner *Pruner) checkAnnotationsRemovedNamespaces(ctx context.Context) ([]
 	return uniqueCronName, err
 }
 
+func diffLastAppliedPruneConfig(ctx context.Context, k kubernetes.Interface, defaultconf v1alpha1.Prune, ns *corev1.Namespace) (bool, error) {
+	var oldAnnotations string
+	pruneAnnotationList := []string{pruneSkip, pruneResources, pruneStrategy, pruneKeep, pruneKeepSince, pruneSchedule}
+
+	annotations := ns.GetAnnotations()
+	if annotations == nil {
+		annotations = make(map[string]string)
+	}
+
+	for _, key := range pruneAnnotationList {
+		if val, ok := annotations[key]; ok {
+			oldAnnotations = oldAnnotations + fmt.Sprint(key) + fmt.Sprint(val)
+		}
+	}
+
+	// this will take care of the prune related annotations in the namespace
+	if oldAnnotations != "" {
+		computedAnnotationsHash, err := ComputeHashOf(oldAnnotations)
+		if err != nil {
+			return changed, err
+		}
+		return checkConfigChangeUpdateHashInNamespace(ctx, k, ns, annotations, computedAnnotationsHash)
+	}
+
+	// if there are no annotations but, change in default config
+	if defaultconf.IsEmpty() {
+		return changed, nil
+	}
+
+	computedDefaultConfigHash, err := ComputeHashOf(defaultconf)
+	if err != nil {
+		return changed, err
+	}
+
+	return checkConfigChangeUpdateHashInNamespace(ctx, k, ns, annotations, computedDefaultConfigHash)
+}
+
+func checkConfigChangeUpdateHashInNamespace(ctx context.Context, k kubernetes.Interface, ns *corev1.Namespace, annotations map[string]string, currentComputedHash string) (bool, error) {
+	lastHash := annotations[pruneLastAppliedHash]
+	if lastHash != currentComputedHash {
+		annotations[pruneLastAppliedHash] = currentComputedHash
+		ns.SetAnnotations(annotations)
+		_, err := k.CoreV1().Namespaces().Update(ctx, ns, v1.UpdateOptions{})
+		return changed, err
+	}
+	return unchanged, nil
+}
+
 func stringToUint(num string) *uint {
 	keep, _ := strconv.Atoi(num)
 	uintKeep := uint(keep)
