@@ -49,27 +49,51 @@ func CreateAddonCR(instance v1alpha1.TektonComponent, client operatorv1alpha1.Op
 func ensureTektonAddonExists(clients op.TektonAddonInterface, config *v1alpha1.TektonConfig) (*v1alpha1.TektonAddon, error) {
 	taCR, err := GetAddon(clients, common.AddonResourceName)
 	if err == nil {
-		// If addon params are changed in TektonConfig spec, then only update the Addon CR
+		// if the addon spec is changed then update the instance
+		updated := false
+
+		if config.Spec.TargetNamespace != taCR.Spec.TargetNamespace {
+			taCR.Spec.TargetNamespace = config.Spec.TargetNamespace
+			updated = true
+		}
+
 		if !reflect.DeepEqual(config.Spec.Addon.Params, taCR.Spec.Params) {
 			taCR.Spec.Params = config.Spec.Addon.Params
-			updated, err := clients.Update(context.TODO(), taCR, metav1.UpdateOptions{})
-			if err != nil {
-				return nil, err
-			}
-			return updated, nil
+			updated = true
 		}
+
+		if !reflect.DeepEqual(taCR.Spec.Config, config.Spec.Config) {
+			taCR.Spec.Config = config.Spec.Config
+			updated = true
+		}
+
+		if taCR.ObjectMeta.OwnerReferences == nil {
+			ownerRef := *metav1.NewControllerRef(config, config.GroupVersionKind())
+			taCR.ObjectMeta.OwnerReferences = []metav1.OwnerReference{ownerRef}
+			updated = true
+		}
+
+		if updated {
+			return clients.Update(context.TODO(), taCR, metav1.UpdateOptions{})
+		}
+
 		return taCR, err
 	}
+
+	ownerRef := *metav1.NewControllerRef(config, config.GroupVersionKind())
+
 	if apierrs.IsNotFound(err) {
 		taCR = &v1alpha1.TektonAddon{
 			ObjectMeta: metav1.ObjectMeta{
-				Name: common.AddonResourceName,
+				Name:            common.AddonResourceName,
+				OwnerReferences: []metav1.OwnerReference{ownerRef},
 			},
 			Spec: v1alpha1.TektonAddonSpec{
 				CommonSpec: v1alpha1.CommonSpec{
 					TargetNamespace: config.Spec.TargetNamespace,
 				},
 				Params: config.Spec.Addon.Params,
+				Config: config.Spec.Config,
 			},
 		}
 		return clients.Create(context.TODO(), taCR, metav1.CreateOptions{})
@@ -103,6 +127,13 @@ func waitForTektonAddonState(clients op.TektonAddonInterface, name string,
 
 // isTektonAddonReady will check the status conditions of the TektonAddon and return true if the TektonAddon is ready.
 func isTektonAddonReady(s *v1alpha1.TektonAddon, err error) (bool, error) {
+	upgradePending, errInternal := common.CheckUpgradePending(s)
+	if err != nil {
+		return false, errInternal
+	}
+	if upgradePending {
+		return false, v1alpha1.DEPENDENCY_UPGRADE_PENDING_ERR
+	}
 	return s.Status.IsReady(), err
 }
 
